#include "TCPSocket.h"
#include <iostream>
#include <string.h>

using namespace std;

/**
 * Konstruktor TCPSocket() vytvori soket, ktory umozni komunikaciu po sieti prostrednictvom protokolu TCP.
 */

TCPSocket::TCPSocket()
{
#ifdef WIN32
	WSAData wsaData;
	if (WSAStartup(MAKEWORD(2, 2), &wsaData) != -1)
	{
		cout << "Agent spustil WSA." << endl;
	}
	else
	{
		cerr << "Agent nedokazal spustit WSA." << endl;
	}
#endif
	fileDescriptor = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (fileDescriptor != -1)
	{
		cout << "Agent vytvoril soket." << endl;
	}
	else
	{
		cerr << "Agent nedokazal vytvorit soket." << endl;
	}
}

/**
 * Metoda connect() pripoji soket na server.
 * @param port - Port v "network byte order", na ktorom je mozne komunikovat so serverom.
 * @param address - Adresa servera v "network byte order".
 * @return Ak sa soket pripojil na server, tak true inak false.
 */

bool TCPSocket::connect(unsigned short port, unsigned long address)
{
	struct sockaddr_in server;
	server.sin_family = AF_INET;
	server.sin_port = port;
	server.sin_addr.s_addr = address;
	if (::connect(fileDescriptor, (sockaddr *) &server, sizeof(server)) != -1)
	{
		cout << "Agent sa pripojil na server." << endl;
		return true;
	}
	cerr << "Agent sa nedokazal pripojit na server." << endl;
	return false;
}

/**
 * Ak je dlzka posielanej spravy vacsia ako 0, tak metoda send() posle cez soket data, ktore pozostavaju z dlzky posielanej spravy v "network byte order" a zo samotnej posielanej spravy bez ukoncovacieho znaku.
 * @param sendingMessage - Posielana sprava, ktora predpoklada pred svojim zaciatkom 4 volne byty.
 * @return Ak je dlzka posielanej spravy rovna 0 alebo ak sa cez soket poslali data, tak true inak false.
 */

bool TCPSocket::send(char *sendingMessage)
{
	int sendingMessageLength = (int) strlen(sendingMessage);
	if (sendingMessageLength == 0)
	{
		return true;
	}
	int sendingMessageLength2 = htonl(sendingMessageLength);
	char *sendingData = sendingMessage - 4;
	memcpy(sendingData, &sendingMessageLength2, 4);
	if (::send(fileDescriptor, sendingData, 4 + sendingMessageLength, 0) != -1)
	{
		return true;
	}
	cerr << "Agent nedokazal poslat data." << endl;
	return false;
}

/**
 * Pokial na sokete nie su prijimane data, tak metoda select() caka, kym prijimane data nepridu alebo nevyprsi casovy interval.
 * @param seconds - Sekundy casoveho intervalu.
 * @param microseconds - Mikrosekundy casoveho intervalu.
 * @return Ak do vyprsania casoveho intervalu su na sokete prijimane data, tak metoda okamzite vrati true inak false.
 */

bool TCPSocket::select(int seconds, int microseconds)
{
	struct timeval timeout;
	timeout.tv_sec = seconds;
	timeout.tv_usec = microseconds;
	fd_set fileDescriptorSet;
	FD_ZERO(&fileDescriptorSet);
	FD_SET(fileDescriptor, &fileDescriptorSet);
	return ::select((int) fileDescriptor + 1, &fileDescriptorSet, 0, 0, &timeout) > 0;
}

/**
 * Ak je maximalna dlzka prijimanej spravy vacsia ako dlzka prijimanej spravy, tak metoda receive() prijme cez soket data, ktore pozostavaju z dlzky prijimanej spravy v "network byte order" a zo samotnej prijimanej spravy bez ukoncovacieho znaku.
 * @param receivingMessage - Vystupna prijimana sprava, ktora predpoklada pred svojim zaciatkom 4 volne byty.
 * @param receivingMessageSize - Maximalna dlzka prijimanej spravy.
 * @return Ak je maximalna dlzka prijimanej spravy vacsia ako dlzka prijimanej spravy a cez soket sa prijali data s dlzkou vacsou ako 0, tak true inak false.
 */

bool TCPSocket::receive(char *receivingMessage, int receivingMessageSize)
{
	char *receivingData = receivingMessage - 4;
	int allBytes;
	do
	{
		allBytes = ::recv(fileDescriptor, receivingData, 4, MSG_PEEK);
		switch (allBytes)
		{
			case -1:
				cerr << "Agent nedokazal prijat data." << endl;
				return false;
			case 0:
				cout << "Agent neprijal ziadne data." << endl;
				return false;
		}
	}
	while (allBytes < 4);
	int receivingMessageLength2;
	memcpy(&receivingMessageLength2, receivingData, 4);
	int receivingMessageLength = ntohl(receivingMessageLength2);
	if (receivingMessageSize <= receivingMessageLength)
	{
		cerr << "Agent nedokazal prijat data, pretoze su prilis dlhe." << endl;
		return false;
	}
	int receivingDataLength = 4 + receivingMessageLength;
	allBytes = 0;
	do
	{
		int bytes = ::recv(fileDescriptor, receivingData + allBytes, receivingDataLength - allBytes, 0);
		if (bytes == -1)
		{
			cerr << "Agent nedokazal prijat data." << endl;
			return false;
		}
		allBytes += bytes;
	}
	while (allBytes < receivingDataLength);
	receivingMessage[receivingMessageLength] = '\0';
	return true;
}

/**
 * Destruktor ~TCPSocket() zatvori soket, ktory umoznoval komunikaciu po sieti prostrednictvom protokolu TCP.
 */

TCPSocket::~TCPSocket()
{
#ifdef WIN32
	if (::closesocket(fileDescriptor) != -1)
#else
	if (::close(fileDescriptor) != -1)
#endif
	{
		cout << "Agent zatvoril soket." << endl;
	}
	else
	{
		cerr << "Agent nedokazal zatvorit soket." << endl;
	}
#ifdef WIN32
	if (WSACleanup() != -1)
	{
		cout << "Agent vycistil WSA." << endl;
	}
	else
	{
		cerr << "Agent nedokazal vycistit WSA." << endl;
	}
#endif
}
